@@ -70,7 +70,7 @@ struct _GstOggStream
   gint bitrate;
   guint64 total_time;
   gboolean is_sparse;
-  gboolean audio_clipping;
+  gboolean forbid_start_clamping;
 
   GstCaps *caps;
 
@@ -94,7 +94,6 @@ struct _GstOggStream
   gboolean theora_has_zero_keyoffset;
   /* VP8 stuff */
   gboolean is_vp8;
-  gint64 invisible_count;
   /* opus stuff */
   gint64 first_granpos;
   /* OGM stuff */
@@ -117,8 +116,6 @@ struct _GstOggStream
 gboolean gst_ogg_stream_setup_map (GstOggStream * pad, ogg_packet *packet);
 gboolean gst_ogg_stream_setup_map_from_caps_headers (GstOggStream * pad,
     const GstCaps * caps);
-gboolean gst_ogg_stream_setup_map_from_caps (GstOggStream * pad,
-    const GstCaps * caps);
 GstClockTime gst_ogg_stream_get_end_time_for_granulepos (GstOggStream *pad,
     gint64 granulepos);
 GstClockTime gst_ogg_stream_get_start_time_for_granulepos (GstOggStream *pad,
@@ -136,8 +133,6 @@ gboolean gst_ogg_stream_packet_is_key_frame (GstOggStream *pad, ogg_packet *pack
 gint64 gst_ogg_stream_get_packet_duration (GstOggStream * pad, ogg_packet *packet);
 void gst_ogg_stream_extract_tags (GstOggStream * pad, ogg_packet * packet);
 const char *gst_ogg_stream_get_media_type (GstOggStream * pad);
-GstBuffer *gst_ogg_stream_get_headers (GstOggStream *pad);
-void gst_ogg_stream_update_stats (GstOggStream * pad, ogg_packet * packet);
 
 gboolean gst_ogg_map_parse_fisbone (GstOggStream * pad, const guint8 * data, guint size,
     guint32 * serialno, GstOggSkeleton *type);
