@@ -18,37 +18,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <string.h>
 #include "video.h"
-#ifdef HAVE_GL
-#include <gst/gl/gstglmemory.h>
-#endif
-
-#ifndef GST_DISABLE_GST_DEBUG
-#define GST_CAT_DEFAULT ensure_debug_category()
-static GstDebugCategory *
-ensure_debug_category (void)
-{
-  static gsize cat_gonce = 0;
-
-  if (g_once_init_enter (&cat_gonce)) {
-    gsize cat_done;
-
-    cat_done = (gsize) _gst_debug_category_new ("video-frame-converter", 0,
-        "video-frame-converter object");
-
-    g_once_init_leave (&cat_gonce, cat_done);
-  }
-
-  return (GstDebugCategory *) cat_gonce;
-}
-#else
-#define ensure_debug_category() /* NOOP */
-#endif /* GST_DISABLE_GST_DEBUG */
 
 static gboolean
 caps_are_raw (const GstCaps * caps)
@@ -124,7 +96,7 @@ get_encoder (const GstCaps * caps, GError ** err)
   encoder = gst_element_factory_create (factory, NULL);
 
   GST_INFO ("created encoder element %p, %s", encoder,
-      GST_ELEMENT_NAME (encoder));
+      gst_element_get_name (encoder));
 
 fail:
   if (encoders)
@@ -135,149 +107,14 @@ fail:
   return encoder;
 }
 
-static GstElement *
-build_convert_frame_pipeline_d3d11 (GstElement ** src_element,
-    GstElement ** sink_element, GstCaps * from_caps, GstCaps * to_caps,
-    gboolean is_d3d12, GError ** err)
-{
-  GstElement *pipeline = NULL;
-  GstElement *appsrc = NULL;
-  GstElement *d3d11_convert = NULL;
-  GstElement *d3d11_download = NULL;
-  GstElement *convert = NULL;
-  GstElement *enc = NULL;
-  GstElement *appsink = NULL;
-  GError *error = NULL;
-  const gchar *d3d_conv_name = "d3d11convert";
-  const gchar *d3d_download_name = "d3d11download";
-
-  if (is_d3d12) {
-    d3d_conv_name = "d3d12convert";
-    d3d_download_name = "d3d12download";
-  }
-
-  if (!create_element ("appsrc", &appsrc, &error) ||
-      !create_element (d3d_conv_name, &d3d11_convert, &error) ||
-      !create_element (d3d_download_name, &d3d11_download, &error) ||
-      !create_element ("videoconvert", &convert, &error) ||
-      !create_element ("appsink", &appsink, &error)) {
-    GST_ERROR ("Could not create element");
-    goto failed;
-  }
-
-  if (caps_are_raw (to_caps)) {
-    if (!create_element ("identity", &enc, &error)) {
-      GST_ERROR ("Could not create identity element");
-      goto failed;
-    }
-  } else {
-    enc = get_encoder (to_caps, &error);
-    if (!enc) {
-      GST_ERROR ("Could not create encoder");
-      goto failed;
-    }
-  }
-
-  g_object_set (appsrc, "caps", from_caps, "emit-signals", TRUE,
-      "format", GST_FORMAT_TIME, NULL);
-  g_object_set (appsink, "caps", to_caps, "emit-signals", TRUE, NULL);
-
-  pipeline = gst_pipeline_new ("d3d11-convert-frame-pipeline");
-  gst_bin_add_many (GST_BIN (pipeline), appsrc, d3d11_convert, d3d11_download,
-      convert, enc, appsink, NULL);
-
-  if (!gst_element_link_many (appsrc,
-          d3d11_convert, d3d11_download, convert, enc, appsink, NULL)) {
-    /* Now pipeline takes ownership of all elements, so only top-level
-     * pipeline should be cleared */
-    appsrc = d3d11_convert = convert = enc = appsink = NULL;
-
-    error = g_error_new (GST_CORE_ERROR, GST_CORE_ERROR_NEGOTIATION,
-        "Could not configure pipeline for conversion");
-  }
-
-  *src_element = appsrc;
-  *sink_element = appsink;
-
-  return pipeline;
-
-failed:
-  if (err)
-    *err = error;
-  else
-    g_clear_error (&error);
-
-  gst_clear_object (&pipeline);
-  gst_clear_object (&appsrc);
-  gst_clear_object (&d3d11_convert);
-  gst_clear_object (&d3d11_download);
-  gst_clear_object (&convert);
-  gst_clear_object (&enc);
-  gst_clear_object (&appsink);
-
-  return NULL;
-}
-
-static GstPadProbeReturn
-strip_video_crop_meta (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
-{
-  GstBuffer *buf = gst_pad_probe_info_get_buffer (info);
-  GstVideoCropMeta *cmeta = gst_buffer_get_video_crop_meta (buf);
-
-  if (cmeta != NULL) {
-    /* Make the buffer writable before stripping the meta and
-     * putting the possibly-replaced buffer back into
-     * the pad probe data */
-    GST_DEBUG ("Removing video crop meta from input buffer");
-    buf = gst_buffer_make_writable (buf);
-    gst_buffer_remove_meta (buf, GST_META_CAST (cmeta));
-    GST_PAD_PROBE_INFO_DATA (info) = buf;
-  }
-
-  return GST_PAD_PROBE_OK;
-}
-
 static GstElement *
 build_convert_frame_pipeline (GstElement ** src_element,
-    GstElement ** sink_element, GstCaps * from_caps,
-    GstVideoCropMeta * cmeta, GstCaps * to_caps, GError ** err)
+    GstElement ** sink_element, const GstCaps * from_caps,
+    const GstCaps * to_caps, GError ** err)
 {
-  GstElement *vcrop = NULL, *csp = NULL, *csp2 = NULL, *vscale = NULL;
-  GstElement *src = NULL, *sink = NULL, *encoder = NULL, *pipeline;
-  GstElement *dl = NULL;
-  GstVideoInfo info;
+  GstElement *src = NULL, *csp = NULL, *vscale = NULL;
+  GstElement *sink = NULL, *encoder = NULL, *pipeline;
   GError *error = NULL;
-  GstCapsFeatures *features;
-
-  features = gst_caps_get_features (from_caps, 0);
-  if (features && !gst_caps_features_is_any (features)) {
-    gboolean is_d3d11 =
-        gst_caps_features_contains (features, "memory:D3D11Memory");
-    gboolean is_d3d12 =
-        gst_caps_features_contains (features, "memory:D3D12Memory");
-
-    if (is_d3d11 || is_d3d12) {
-      return build_convert_frame_pipeline_d3d11 (src_element, sink_element,
-          from_caps, to_caps, is_d3d12, err);
-    }
-  }
-#ifdef HAVE_GL
-  if (features &&
-      gst_caps_features_contains (features, GST_CAPS_FEATURE_MEMORY_GL_MEMORY))
-    if (!create_element ("gldownload", &dl, &error))
-      goto no_elements;
-#endif
-
-  if (cmeta) {
-    if (!create_element ("videocrop", &vcrop, &error)) {
-      g_error_free (error);
-      g_warning
-          ("build_convert_frame_pipeline: Buffer has crop metadata but videocrop element is not found. Cropping will be disabled");
-    } else {
-      if (!create_element ("videoconvert", &csp2, &error))
-        goto no_elements;
-    }
-  }
 
   /* videoscale is here to correct for the pixel-aspect-ratio for us */
   GST_DEBUG ("creating elements");
@@ -296,75 +133,15 @@ build_convert_frame_pipeline (GstElement ** src_element,
 
   GST_DEBUG ("adding elements");
   gst_bin_add_many (GST_BIN (pipeline), src, csp, vscale, sink, NULL);
-  if (vcrop)
-    gst_bin_add_many (GST_BIN (pipeline), vcrop, csp2, NULL);
-  if (dl)
-    gst_bin_add (GST_BIN (pipeline), dl);
 
-  /* set input and output caps */
-  g_object_set (src, "caps", from_caps, "emit-signals", TRUE,
-      "format", GST_FORMAT_TIME, NULL);
-  g_object_set (sink, "caps", to_caps, "emit-signals", TRUE, NULL);
+  /* set caps */
+  g_object_set (src, "caps", from_caps, NULL);
+  g_object_set (sink, "caps", to_caps, NULL);
 
-  if (vcrop) {
-    gst_video_info_from_caps (&info, from_caps);
-    g_object_set (vcrop, "left", cmeta->x, NULL);
-    g_object_set (vcrop, "top", cmeta->y, NULL);
-    g_object_set (vcrop, "right",
-        GST_VIDEO_INFO_WIDTH (&info) - (cmeta->x + cmeta->width), NULL);
-    g_object_set (vcrop, "bottom",
-        GST_VIDEO_INFO_HEIGHT (&info) - (cmeta->y + cmeta->height), NULL);
-
-    /* Because we are asking videocrop to apply the cropping explicitly,
-     * we need to give it a buffer without crop meta, or the crop amounts end up
-     * getting applied twice. Add a probe to strip the meta and use videocrop's
-     * properties */
-    GstPad *sink = gst_element_get_static_pad (vcrop, "sink");
-    g_assert (sink != NULL);
-    gst_pad_add_probe (sink, GST_PAD_PROBE_TYPE_BUFFER,
-        strip_video_crop_meta, NULL, NULL);
-    gst_object_unref (sink);
-
-    GST_DEBUG ("crop meta [x,y,width,height]: %d %d %d %d", cmeta->x, cmeta->y,
-        cmeta->width, cmeta->height);
-
-    /* FIXME: linking is still way too expensive, profile this properly */
-    if (!dl) {
-      GST_DEBUG ("linking src->csp2");
-      if (!gst_element_link_pads (src, "src", csp2, "sink"))
-        goto link_failed;
-    } else {
-      GST_DEBUG ("linking src->dl");
-      if (!gst_element_link_pads (src, "src", dl, "sink"))
-        goto link_failed;
-
-      GST_DEBUG ("linking dl->csp2");
-      if (!gst_element_link_pads (dl, "src", csp2, "sink"))
-        goto link_failed;
-    }
-
-    GST_DEBUG ("linking csp2->vcrop");
-    if (!gst_element_link_pads (csp2, "src", vcrop, "sink"))
-      goto link_failed;
-
-    GST_DEBUG ("linking vcrop->csp");
-    if (!gst_element_link_pads (vcrop, "src", csp, "sink"))
-      goto link_failed;
-  } else {
-    GST_DEBUG ("linking src->csp");
-    if (!dl) {
-      if (!gst_element_link_pads (src, "src", csp, "sink"))
-        goto link_failed;
-    } else {
-      GST_DEBUG ("linking src->dl");
-      if (!gst_element_link_pads (src, "src", dl, "sink"))
-        goto link_failed;
-
-      GST_DEBUG ("linking dl->csp");
-      if (!gst_element_link_pads (dl, "src", csp, "sink"))
-        goto link_failed;
-    }
-  }
+  /* FIXME: linking is still way too expensive, profile this properly */
+  GST_DEBUG ("linking src->csp");
+  if (!gst_element_link_pads (src, "src", csp, "sink"))
+    goto link_failed;
 
   GST_DEBUG ("linking csp->vscale");
   if (!gst_element_link_pads_full (csp, "src", vscale, "sink",
@@ -392,6 +169,9 @@ build_convert_frame_pipeline (GstElement ** src_element,
       goto link_failed;
   }
 
+  g_object_set (src, "emit-signals", TRUE, NULL);
+  g_object_set (sink, "emit-signals", TRUE, NULL);
+
   *src_element = src;
   *sink_element = sink;
 
@@ -413,12 +193,8 @@ no_elements:
   {
     if (src)
       gst_object_unref (src);
-    if (vcrop)
-      gst_object_unref (vcrop);
     if (csp)
       gst_object_unref (csp);
-    if (csp2)
-      gst_object_unref (csp2);
     if (vscale)
       gst_object_unref (vscale);
     if (sink)
@@ -433,11 +209,7 @@ no_elements:
 no_pipeline:
   {
     gst_object_unref (src);
-    if (vcrop)
-      gst_object_unref (vcrop);
     gst_object_unref (csp);
-    if (csp2)
-      gst_object_unref (csp2);
     gst_object_unref (vscale);
     gst_object_unref (sink);
 
@@ -472,7 +244,7 @@ link_failed:
  *
  * The width, height and pixel-aspect-ratio can also be specified in the output caps.
  *
- * Returns: (nullable) (transfer full): The converted #GstSample, or %NULL if an error happened (in which case @err
+ * Returns: The converted #GstSample, or %NULL if an error happened (in which case @err
  * will point to the #GError).
  */
 GstSample *
@@ -498,6 +270,7 @@ gst_video_convert_sample (GstSample * sample, const GstCaps * to_caps,
   from_caps = gst_sample_get_caps (sample);
   g_return_val_if_fail (from_caps != NULL, NULL);
 
+
   to_caps_copy = gst_caps_new_empty ();
   n = gst_caps_get_size (to_caps);
   for (i = 0; i < n; i++) {
@@ -509,8 +282,7 @@ gst_video_convert_sample (GstSample * sample, const GstCaps * to_caps,
   }
 
   pipeline =
-      build_convert_frame_pipeline (&src, &sink, from_caps,
-      gst_buffer_get_video_crop_meta (buf), to_caps_copy, &err);
+      build_convert_frame_pipeline (&src, &sink, from_caps, to_caps_copy, &err);
   if (!pipeline)
     goto no_pipeline;
 
@@ -518,9 +290,7 @@ gst_video_convert_sample (GstSample * sample, const GstCaps * to_caps,
    * appsrc, this should preroll the converted buffer in appsink */
   GST_DEBUG ("running conversion pipeline to caps %" GST_PTR_FORMAT,
       to_caps_copy);
-  if (gst_element_set_state (pipeline,
-          GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE)
-    goto state_change_failed;
+  gst_element_set_state (pipeline, GST_STATE_PAUSED);
 
   /* feed buffer in appsrc */
   GST_DEBUG ("feeding buffer %p, size %" G_GSIZE_FORMAT ", caps %"
@@ -583,7 +353,6 @@ gst_video_convert_sample (GstSample * sample, const GstCaps * to_caps,
 
   /* ERRORS */
 no_pipeline:
-state_change_failed:
   {
     gst_caps_unref (to_caps_copy);
 
@@ -598,7 +367,6 @@ state_change_failed:
 
 typedef struct
 {
-  gint ref_count;
   GMutex mutex;
   GstElement *pipeline;
   GstVideoConvertSampleCallback callback;
@@ -606,60 +374,57 @@ typedef struct
   GDestroyNotify destroy_notify;
   GMainContext *context;
   GstSample *sample;
-  GSource *timeout_source;
+  //GstBuffer *buffer;
+  gulong timeout_id;
   gboolean finished;
-
-  /* Results */
-  GstSample *converted_sample;
-  GError *error;
 } GstVideoConvertSampleContext;
 
-static GstVideoConvertSampleContext *
-gst_video_convert_frame_context_ref (GstVideoConvertSampleContext * ctx)
+typedef struct
 {
-  g_atomic_int_inc (&ctx->ref_count);
+  GstVideoConvertSampleCallback callback;
+  GstSample *sample;
+  //GstBuffer *buffer;
+  GError *error;
+  gpointer user_data;
+  GDestroyNotify destroy_notify;
 
-  return ctx;
+  GstVideoConvertSampleContext *context;
+} GstVideoConvertSampleCallbackContext;
+
+static void
+gst_video_convert_frame_context_free (GstVideoConvertSampleContext * ctx)
+{
+  /* Wait until all users of the mutex are done */
+  g_mutex_lock (&ctx->mutex);
+  g_mutex_unlock (&ctx->mutex);
+  g_mutex_clear (&ctx->mutex);
+  if (ctx->timeout_id)
+    g_source_remove (ctx->timeout_id);
+  //if (ctx->buffer)
+  //  gst_buffer_unref (ctx->buffer);
+  if (ctx->sample)
+    gst_sample_unref (ctx->sample);
+  g_main_context_unref (ctx->context);
+
+  gst_element_set_state (ctx->pipeline, GST_STATE_NULL);
+  gst_object_unref (ctx->pipeline);
+
+  g_slice_free (GstVideoConvertSampleContext, ctx);
 }
 
 static void
-gst_video_convert_frame_context_unref (GstVideoConvertSampleContext * ctx)
+    gst_video_convert_frame_callback_context_free
+    (GstVideoConvertSampleCallbackContext * ctx)
 {
-  if (!g_atomic_int_dec_and_test (&ctx->ref_count))
-    return;
-
-  g_mutex_clear (&ctx->mutex);
-  if (ctx->timeout_source)
-    g_source_destroy (ctx->timeout_source);
-  if (ctx->sample)
-    gst_sample_unref (ctx->sample);
-  if (ctx->converted_sample)
-    gst_sample_unref (ctx->converted_sample);
-  g_clear_error (&ctx->error);
-  g_main_context_unref (ctx->context);
-
-  /* The pipeline was already destroyed in finish() earlier and we
-   * must not end up here without finish() being called */
-  g_warn_if_fail (ctx->pipeline == NULL);
-
-  g_free (ctx);
+  if (ctx->context)
+    gst_video_convert_frame_context_free (ctx->context);
+  g_slice_free (GstVideoConvertSampleCallbackContext, ctx);
 }
 
 static gboolean
-convert_frame_dispatch_callback (GstVideoConvertSampleContext * ctx)
+convert_frame_dispatch_callback (GstVideoConvertSampleCallbackContext * ctx)
 {
-  GstSample *sample;
-  GError *error;
-
-  g_return_val_if_fail (ctx->converted_sample != NULL
-      || ctx->error != NULL, FALSE);
-
-  sample = ctx->converted_sample;
-  error = ctx->error;
-  ctx->converted_sample = NULL;
-  ctx->error = NULL;
-
-  ctx->callback (sample, error, ctx->user_data);
+  ctx->callback (ctx->sample, ctx->error, ctx->user_data);
 
   if (ctx->destroy_notify)
     ctx->destroy_notify (ctx->user_data);
@@ -667,52 +432,34 @@ convert_frame_dispatch_callback (GstVideoConvertSampleContext * ctx)
   return FALSE;
 }
 
-static void
-convert_frame_stop_pipeline (GstElement * element, gpointer user_data)
-{
-  gst_element_set_state (element, GST_STATE_NULL);
-}
-
 static void
 convert_frame_finish (GstVideoConvertSampleContext * context,
     GstSample * sample, GError * error)
 {
   GSource *source;
+  GstVideoConvertSampleCallbackContext *ctx;
 
-  g_return_if_fail (!context->finished);
-  g_return_if_fail (sample != NULL || error != NULL);
+  if (context->timeout_id)
+    g_source_remove (context->timeout_id);
+  context->timeout_id = 0;
 
-  context->finished = TRUE;
-  context->converted_sample = sample;
-  context->error = error;
-
-  if (context->timeout_source)
-    g_source_destroy (context->timeout_source);
-  context->timeout_source = NULL;
+  ctx = g_slice_new (GstVideoConvertSampleCallbackContext);
+  ctx->callback = context->callback;
+  ctx->user_data = context->user_data;
+  ctx->destroy_notify = context->destroy_notify;
+  ctx->sample = sample;
+  //ctx->buffer = buffer;
+  ctx->error = error;
+  ctx->context = context;
 
   source = g_timeout_source_new (0);
   g_source_set_callback (source,
-      (GSourceFunc) convert_frame_dispatch_callback,
-      gst_video_convert_frame_context_ref (context),
-      (GDestroyNotify) gst_video_convert_frame_context_unref);
+      (GSourceFunc) convert_frame_dispatch_callback, ctx,
+      (GDestroyNotify) gst_video_convert_frame_callback_context_free);
   g_source_attach (source, context->context);
   g_source_unref (source);
 
-  /* Asynchronously stop the pipeline here: this will set its
-   * state to NULL and get rid of its last reference, which in turn
-   * will get rid of all remaining references to our context and free
-   * it too. We can't do this directly here as we might be called from
-   * a streaming thread.
-   *
-   * We don't use the main loop here because the user might shut down it
-   * immediately after getting the result of the conversion above.
-   */
-  if (context->pipeline) {
-    gst_object_call_async (GST_OBJECT_CAST (context->pipeline),
-        (GstObjectCallAsyncFunc) convert_frame_stop_pipeline, NULL);
-    gst_object_unref (context->pipeline);
-    context->pipeline = NULL;
-  }
+  context->finished = TRUE;
 }
 
 static gboolean
@@ -798,11 +545,11 @@ convert_frame_need_data_callback (GstElement * src, guint size,
     convert_frame_finish (context, NULL, error);
   }
 
-done:
-  g_mutex_unlock (&context->mutex);
-
   g_signal_handlers_disconnect_by_func (src, convert_frame_need_data_callback,
       context);
+
+done:
+  g_mutex_unlock (&context->mutex);
 }
 
 static GstFlowReturn
@@ -825,12 +572,12 @@ convert_frame_new_preroll_callback (GstElement * sink,
   }
   convert_frame_finish (context, sample, error);
 
-done:
-  g_mutex_unlock (&context->mutex);
-
   g_signal_handlers_disconnect_by_func (sink, convert_frame_need_data_callback,
       context);
 
+done:
+  g_mutex_unlock (&context->mutex);
+
   return GST_FLOW_OK;
 }
 
@@ -849,7 +596,7 @@ done:
  *
  * The width, height and pixel-aspect-ratio can also be specified in the output caps.
  *
- * @callback will be called after conversion, when an error occurred or if conversion didn't
+ * @callback will be called after conversion, when an error occured or if conversion didn't
  * finish after @timeout. @callback will always be called from the thread default
  * %GMainContext, see g_main_context_get_thread_default(). If GLib before 2.22 is used,
  * this will always be the global default main context.
@@ -898,87 +645,70 @@ gst_video_convert_sample_async (GstSample * sample,
     gst_caps_append_structure (to_caps_copy, s);
   }
 
-  /* There's a reference cycle between the context and the pipeline, which is
-   * broken up once the finish() is called on the context. At latest when the
-   * timeout triggers the context will be freed */
-  ctx = g_new0 (GstVideoConvertSampleContext, 1);
-  ctx->ref_count = 1;
+  pipeline =
+      build_convert_frame_pipeline (&src, &sink, from_caps, to_caps_copy,
+      &error);
+  if (!pipeline)
+    goto no_pipeline;
+
+  bus = gst_element_get_bus (pipeline);
+
+  ctx = g_slice_new0 (GstVideoConvertSampleContext);
   g_mutex_init (&ctx->mutex);
+  //ctx->buffer = gst_buffer_ref (buf);
   ctx->sample = gst_sample_ref (sample);
   ctx->callback = callback;
   ctx->user_data = user_data;
   ctx->destroy_notify = destroy_notify;
   ctx->context = g_main_context_ref (context);
   ctx->finished = FALSE;
-
-  pipeline =
-      build_convert_frame_pipeline (&src, &sink, from_caps,
-      gst_buffer_get_video_crop_meta (buf), to_caps_copy, &error);
-  if (!pipeline)
-    goto no_pipeline;
   ctx->pipeline = pipeline;
 
-  bus = gst_element_get_bus (pipeline);
-
   if (timeout != GST_CLOCK_TIME_NONE) {
-    ctx->timeout_source = g_timeout_source_new (timeout / GST_MSECOND);
-    g_source_set_callback (ctx->timeout_source,
-        (GSourceFunc) convert_frame_timeout_callback,
-        gst_video_convert_frame_context_ref (ctx),
-        (GDestroyNotify) gst_video_convert_frame_context_unref);
-    g_source_attach (ctx->timeout_source, context);
+    source = g_timeout_source_new (timeout / GST_MSECOND);
+    g_source_set_callback (source,
+        (GSourceFunc) convert_frame_timeout_callback, ctx, NULL);
+    ctx->timeout_id = g_source_attach (source, context);
+    g_source_unref (source);
   }
 
-  g_signal_connect_data (src, "need-data",
-      G_CALLBACK (convert_frame_need_data_callback),
-      gst_video_convert_frame_context_ref (ctx),
-      (GClosureNotify) gst_video_convert_frame_context_unref, 0);
-  g_signal_connect_data (sink, "new-preroll",
-      G_CALLBACK (convert_frame_new_preroll_callback),
-      gst_video_convert_frame_context_ref (ctx),
-      (GClosureNotify) gst_video_convert_frame_context_unref, 0);
+  g_signal_connect (src, "need-data",
+      G_CALLBACK (convert_frame_need_data_callback), ctx);
+  g_signal_connect (sink, "new-preroll",
+      G_CALLBACK (convert_frame_new_preroll_callback), ctx);
 
   source = gst_bus_create_watch (bus);
   g_source_set_callback (source, (GSourceFunc) convert_frame_bus_callback,
-      gst_video_convert_frame_context_ref (ctx),
-      (GDestroyNotify) gst_video_convert_frame_context_unref);
+      ctx, NULL);
   g_source_attach (source, context);
   g_source_unref (source);
+
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
   gst_object_unref (bus);
-
-  if (gst_element_set_state (pipeline,
-          GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE)
-    goto state_change_failed;
-
   gst_caps_unref (to_caps_copy);
 
-  gst_video_convert_frame_context_unref (ctx);
-
   return;
   /* ERRORS */
 no_pipeline:
   {
+    GstVideoConvertSampleCallbackContext *ctx;
+    GSource *source;
+
     gst_caps_unref (to_caps_copy);
 
-    g_mutex_lock (&ctx->mutex);
-    convert_frame_finish (ctx, NULL, error);
-    g_mutex_unlock (&ctx->mutex);
-    gst_video_convert_frame_context_unref (ctx);
+    ctx = g_slice_new0 (GstVideoConvertSampleCallbackContext);
+    ctx->callback = callback;
+    ctx->user_data = user_data;
+    ctx->destroy_notify = destroy_notify;
+    ctx->sample = NULL;
+    ctx->error = error;
 
-    return;
-  }
-state_change_failed:
-  {
-    gst_caps_unref (to_caps_copy);
-
-    error = g_error_new (GST_CORE_ERROR, GST_CORE_ERROR_STATE_CHANGE,
-        "failed to change state to PLAYING");
-
-    g_mutex_lock (&ctx->mutex);
-    convert_frame_finish (ctx, NULL, error);
-    g_mutex_unlock (&ctx->mutex);
-    gst_video_convert_frame_context_unref (ctx);
-
-    return;
+    source = g_timeout_source_new (0);
+    g_source_set_callback (source,
+        (GSourceFunc) convert_frame_dispatch_callback, ctx,
+        (GDestroyNotify) gst_video_convert_frame_callback_context_free);
+    g_source_attach (source, context);
+    g_source_unref (source);
   }
 }
