@@ -62,6 +62,8 @@ G_BEGIN_DECLS
  * @GST_VIDEO_TEST_SRC_BALL: Moving ball
  * @GST_VIDEO_TEST_SRC_SMPTE100: SMPTE test pattern (100% color bars)
  * @GST_VIDEO_TEST_SRC_BAR: Bar with foreground color
+ * @GST_VIDEO_TEST_SRC_PINWHEEL: Pinwheel
+ * @GST_VIDEO_TEST_SRC_SPOKES: Spokes
  *
  * The test pattern to produce.
  *
@@ -103,7 +105,9 @@ typedef enum {
   GST_VIDEO_TEST_SRC_SOLID,
   GST_VIDEO_TEST_SRC_BALL,
   GST_VIDEO_TEST_SRC_SMPTE100,
-  GST_VIDEO_TEST_SRC_BAR
+  GST_VIDEO_TEST_SRC_BAR,
+  GST_VIDEO_TEST_SRC_PINWHEEL,
+  GST_VIDEO_TEST_SRC_SPOKES
 } GstVideoTestSrcPattern;
 
 typedef struct _GstVideoTestSrc GstVideoTestSrc;
