@@ -203,4 +203,19 @@ xmp_config_suite (void)
   return s;
 }
 
-GST_CHECK_MAIN (xmp_config);
+int
+main (int argc, char **argv)
+{
+  int nf;
+
+  Suite *s = xmp_config_suite ();
+  SRunner *sr = srunner_create (s);
+
+  gst_check_init (&argc, &argv);
+
+  srunner_run_all (sr, CK_NORMAL);
+  nf = srunner_ntests_failed (sr);
+  srunner_free (sr);
+
+  return nf;
+}
